#!/bin/bash
# Copyright (c) 2018 Cloudera, Inc. All rights reserved.

# this script calls the platform specific package management code to determine the
# base url for cloudera-manager-server for the purpose of installing the corresponding
# agent package.
# the output of this script is "Unknown" if it couldn't discover the URL

# Distro detection using /etc/*-release files.
#
# Modern distros expose the lsb_release binary for distro detection, but not
# only is it not installed by default on SLES11, but lsb_release can't be used
# to differentiate between vanilla SLES11 and SLES11 SP1. The /etc/*-release
# files system is older, but it's reliable and always available.
detect_distro()
{
    local RHEL_FILE=/etc/redhat-release
    local SLES_FILE=/etc/os-release
    local UBUNTU_FILE=/etc/lsb-release
    local DEBIAN_FILE=/etc/debian_version


    if [[ -f $RHEL_FILE ]]; then
        OS="RHEL"

        if [[ -z $RHEL_VERSION ]]; then
            grep Santiago $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=6
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'CentOS Linux release 6' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=6
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'CentOS release 6' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=6
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'Scientific Linux release 6' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=6
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep Maipo $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=7
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'CentOS Linux release 7' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=7
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'Scientific Linux release 7' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                RHEL_VERSION=7
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep Ootpa $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                echo $RHEL_FILE '==>' RHEL 8
                RHEL_VERSION=8
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'CentOS Linux release 8' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                echo $RHEL_FILE '==>' CentOS 8
                RHEL_VERSION=8
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'Scientific Linux release 8' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                echo $RHEL_FILE '==>' Scientific Linux 8
                RHEL_VERSION=8
            fi
        fi
        if [[ -z $RHEL_VERSION ]]; then
            grep 'Red Hat Enterprise Linux release' $RHEL_FILE
            if [[ $? -eq 0 ]]; then
                echo $RHEL_FILE '==>' Enterprise Linux 9
                RHEL_VERSION=9
            fi
        fi
    elif [[ -f $UBUNTU_FILE ]]; then
        OS="Ubuntu"
        grep 'Ubuntu' $UBUNTU_FILE
        if [[ $? -eq 0 ]]; then
            CODE_LINE=$(grep DISTRIB_CODENAME $UBUNTU_FILE)
            if [[ $? -eq 0 ]]; then
                CODENAME=$(echo $CODE_LINE | cut -d = -f 2)
            fi
        fi
    elif [[ -f $DEBIAN_FILE ]]; then
        OS="Debian"
        # Debian *must* come after ubuntu as ubuntu provides a debian_version
        # file too
        CODENAME=$(cat $DEBIAN_FILE)
    elif [[ -f $SLES_FILE ]]; then
        OS="SLES"
        SLES_SP_VERSION=$(grep "SUSE Linux Enterprise Server" /etc/os-release | sed -r 's/.*Server\s(\w+\s*\w*)"/\1/')
        if [[ ! -z "$SLES_SP_VERSION" ]]; then
          echo $SLES_FILE '==>' $SLES_SP_VERSION
        fi
    fi
}

{
  detect_distro

  REPO_BASE=
  URL_MATCH=
  if [ $OS == "RHEL" ]; then
      URL_MATCH="\/redhat[0-9][0-9]*\/yum\/*"
      if [ $RHEL_VERSION -ne 8 ] && [ $RHEL_VERSION -ne 9 ] && [ -x "$(command -v find-repos-of-install)" ] && [ -x "$(command -v yum-config-manager)" ]; then
          REPO_NAME=$(find-repos-of-install cloudera-manager-server | grep "from repo" | sed 's/.*from repo //')
          REPO_BASE=$(yum-config-manager $REPO_NAME |grep baseurl | sed 's/.*= //')
      else
          REPO_NAME=$(yum info -v cloudera-manager-server | grep "From repo" | sed 's/.*\: //')
          REPO_BASE=$(yum -v repolist $REPO_NAME |grep baseurl | sed 's/.*\: //')
      fi

  elif [[ $OS == "Debian" ]]; then
      REPO_BASE=$(apt-cache policy cloudera-manager-server | grep -A1  "\*\*\*" | grep -v "\*\*\*" | sed -e 's/^.*http/http/' -e 's/ .*//')
      URL_MATCH="\/debian\/apt\/*"

  elif [[ $OS == "Ubuntu" ]]; then
      REPO_BASE=$(apt-cache policy cloudera-manager-server | grep -A1  "\*\*\*" | grep -v "\*\*\*" | sed -e 's/^.*http/http/' -e 's/ .*//')
      URL_MATCH="\/ubuntu[0-9][0-9]*\/apt\/*"

  elif [[ $OS == "SLES" ]]; then
      REPO_NAME=$(zypper info cloudera-manager-server | grep Repository | sed -e 's/.* : //' -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
      REPO_BASE=$(zypper lr "$REPO_NAME" | grep ^URI | sed 's/.*http/http/')
      URL_MATCH="\/sles[0-9][0-9]*\/yum\/*"
  fi

  # Remove the final part of the BASE URL which has the OS name and package type.
  REPO_URL=$(echo $REPO_BASE |  sed "s/${URL_MATCH}$//")

} >&2

echo $REPO_URL
